/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

import com.opensugar.cube.Launcher;
import com.opensugar.cube.Util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

public class MainTest extends Main {

   public static void main( String[] args ) throws Exception {
      new MainTest( args );
   }

   protected MainTest( String[] args ) {
      if ( args.length != 1 ) {
         usage();
      }


      IOException exceptionWhileLoadingProperties = null;
      try {
         loadProperties();
      }
      catch( IOException e ) {
         exceptionWhileLoadingProperties = e;
      }

      FileNotFoundException exceptionWhileSettingSystemStreams = null;
      try {
         setSystemStreams();
      }
      catch ( FileNotFoundException e ) {
         exceptionWhileSettingSystemStreams = e;
      }

      // Now we can start writing to stderr and stdout, NOT before the call to
      // setSystemStreams has returned

      if ( exceptionWhileLoadingProperties != null ) {
         System.err.println( "Error while loading properties: " + exceptionWhileLoadingProperties.toString() );
      }
      if ( exceptionWhileSettingSystemStreams != null ) {
         System.err.println( "Error while setting system streams: " + exceptionWhileSettingSystemStreams.toString() );
      }



      int charge = -1;
      try {
         charge = Integer.valueOf( args[ 0 ] ).intValue();
      }
      catch ( Exception e ) {
         usage();
      }

      System.getProperties().put( "com.opensugar.cube.systemBundleJarFile", "lib/cube0.jar" );

      System.setProperty( "com.opensugar.cube.test", "" + true );
      if ( System.getProperty( "com.opensugar.cube.test.admin.port" ) == null ) {
         System.setProperty( "com.opensugar.cube.test.admin.port", "" + 4000 );
      }
      if ( System.getProperty( "com.opensugar.cube.test.telnet.port" ) == null ) {
         System.setProperty( "com.opensugar.cube.test.telnet.port", "" + 6000 );
      }
      if ( System.getProperty( "com.opensugar.cube.test.http.port" ) == null ) {
         System.setProperty( "com.opensugar.cube.test.http.port", "" + 10000 );
      }
      File refDir = new File( "osc" );
      String baseDirBase = "osc";
      File dir;
      for ( int i = 0; i < charge; i++ ) {
         System.out.println( "####### " + i + " #######" );
         dir = new File( baseDirBase + i );
         if ( !dir.exists() ) {
            Util.recursiveFileCopy( refDir, dir );
         }
         //( new LauncherThread( new File( dir, "bs" ) ) ).start();
         new Launcher( new File( dir, "bs" ), null );
      }

      Object obj = new Object();
      synchronized( obj ) {
         try {
            obj.wait();
         }
         catch ( InterruptedException ignore ) {}
      }
	  
   }

   private class LauncherThread extends Thread {
      private File baseDir;
      public LauncherThread( File baseDir ) {
         this.baseDir = baseDir;
      }
      public void run() {
         new Launcher( baseDir, null );
      }
   }

   private static void usage() {
      System.out.println( "Usage:" );
      System.out.println( "\tjava MainTest n" );
      System.out.println( "\t\t n must be an integer (number of cubes to start)" );

      System.exit( 1 );
   }

}
