/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube;

import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

// A utility class used to store the headers that are specified in a bundle's
// jar file.
// This class keeps a mapping between bundle header names (which are case insensitive)
// and bundle header values.
public class BundleHeaders extends Dictionary {

   // Hashtable of headers:   headerName (in lower-case) --> header value
   private Hashtable headers;

   protected BundleHeaders( Hashtable attributes ) {
      // Create the hashtable of headers.
      headers = new Hashtable();
      if ( attributes != null ) {
         // If the set of attributes is not null, add each attribute to the hashtable of
         // headers.
         Enumeration enum = attributes.keys();
         String key;
         while ( enum.hasMoreElements() ) {
            key = enum.nextElement().toString();
            put( key, attributes.get( key ) );
         }
      }
   }

   // Implementation of the Dictionary interface's get() method.
   public Object get( Object key ) {
      Header header = (Header)headers.get( processKey( key ) );
      if ( header == null ) {
         return null;
      }
      else {
         return header.getValue();
      }
   }

   // Implementation of the Dictionary interface's isEmpty() method.
   public boolean isEmpty() {
      return headers.isEmpty();
   }

   // Implementation of the Dictionary interface's elements() method.
   public Enumeration elements() {
      Enumeration enum = headers.elements();
      Vector vec = new Vector();
      while ( enum.hasMoreElements() ) {
         vec.addElement( ( (Header)enum.nextElement() ).getValue() );
      }
      return vec.elements();
   }

   // Implementation of the Dictionary interface's keys() method.
   //
   // Return keys in the correct case
   public Enumeration keys() {
      Enumeration enum = headers.elements();
      Vector vec = new Vector();
      while ( enum.hasMoreElements() ) {
         vec.addElement( ( (Header)enum.nextElement() ).getKey() );
      }
      return vec.elements();
   }

   // Implementation of the Dictionary interface's put() method.
   public Object put( Object key, Object value ) {
      Object ret = get( key );

      if ( key instanceof String && value instanceof String ) {
         // Process the attribute key, so that header names are case-insensitive.
         headers.put( processKey( key ), new Header( (String)key, (String)value ) );

         return ret;
      }
      else {
         throw new IllegalArgumentException( "Key and value must be of type String" );
      }
   }

   // Implementation of the Dictionary interface's remove() method.
   public Object remove( Object key ) {
      Object ret = get( key );

      headers.remove( processKey( key ) );

      return ret;
   }

   // Implementation of the Dictionary interface's size() method.
   public int size() {
      return headers.size();
   }

   // Process the attribute key, so that header names are case-insensitive.
   // If the key is not a string, throw an IllegalArgumentException.
   // Otherwise, convert the key to lower case.
   private String processKey( Object key ) {
      if ( key instanceof String ) {
         return ( (String)key ).toLowerCase();
      }
      throw new IllegalArgumentException( "Key must be of type String but is of type: " + key.getClass().getName() );
   }

   private class Header {
      private String key;
      private String value;
      public Header( String key, String value ) {
         this.key = key;
         this.value = value;
      }
      public String getKey() {
         return key;
      }
      public String getValue() {
         return value;
      }
   }

}