/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube;

import com.opensugar.cube.serviceRegistry.ServiceRegistryEntry;

import org.osgi.framework.ServiceRegistration;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.Bundle;
import org.osgi.framework.ServiceEvent;

import java.util.Dictionary;

// Implementation of the org.osgi.framework.ServiceRegistration interface
public class ServiceRegistrationImpl implements ServiceRegistration {

   // the sercice reference for this service
   private ServiceReferenceImpl reference;
   // a flag that indicates if this service is still registered
   private boolean registered;

   public ServiceRegistrationImpl( Bundle bundle, String[] classNames, Dictionary properties ) {
      reference = new ServiceReferenceImpl( bundle, classNames, properties );
      registered = true;
   }

   // Return the reference for this service if the service is still registered.
   // If the service has been unregistered, throw an IllegalStateException.
   public ServiceReference getReference() {
      // Check if the service is still registered and throw an IllegalStateException if it is not.
      checkStillRegistered();

      return reference;
   }

   // Set this service's properties if the service is still registered.
   // If the service has been unregistered, throw an IllegalStateException.
   public void setProperties( Dictionary properties ) {
      // null properties argument is allowed

      // Check if the service is still registered and throw an IllegalStateException if it is not.
      checkStillRegistered();

      // Set the service properties.
      reference.setProperties( properties );

      // Fire a service modified event
      ( (BundleImpl)reference.getBundle() ).getCube().fireServiceEvent( ServiceEvent.MODIFIED, reference );
   }

   public void unregister() {
      // Check if the service is still registered and throw an IllegalStateException if it is not.
      checkStillRegistered();

      registered = false;
      // Do not remove service registration from service registry because service properties
      // and property keys still need to be accessible after service is unregistered

      // Fire service unregistering event
      BundleImpl bundle = (BundleImpl)reference.getBundle();
      Object serviceClass = reference.getProperty( "objectClass" );
      if ( serviceClass instanceof Object[] ) {
         serviceClass = ( (Object[])serviceClass )[ 0 ];
      }
      ( bundle.getCube() ).log( AbstractCube.LOG_DEBUG, bundle.getLocationWithoutAdminPermissionCheck() + " unregistering " + serviceClass );
      bundle.getCube().fireServiceEvent( ServiceEvent.UNREGISTERING, reference );

      // Let service reference know the service has been unregistered so that it knows
      // to return null when its getBundle method is called
      reference.serviceUnregistered();

      // For each bundle whose use count for this service is greater than zero,
      //    1. set the bundle's use count for this service to zero
      //    2. if the service was registered with a service factory, call the factory's
      //       unget method to release the service object for the bundle
      ServiceRegistryEntry serviceRegistryEntry = bundle.getCube().getServiceRegistry().getServiceRegistryEntry( this );
      serviceRegistryEntry.serviceUnregistered();
   }

// *****************************************************************************

   // Check if this service is still registered.
   private void checkStillRegistered() {
      if ( !registered ) {
         throw new IllegalStateException( "Service has already been unregistered" );
      }
   }

}