/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT
 *
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org).
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License.
 *
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 *
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube;

//import com.opensugar.cube.permissionAdmin.PermissionAdminImpl;

import org.osgi.framework.Constants;
import org.osgi.framework.BundleException;
import org.osgi.service.packageadmin.PackageAdmin;
import org.osgi.service.permissionadmin.PermissionAdmin;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class SystemBundle extends BundleImpl {

   private BundleContextImpl bundleContext;
   private int state;

   protected SystemBundle( AbstractCube cube, File bundleJarFile ) throws IOException, ClassNotFoundException, IllegalAccessException, InstantiationException {
      super( cube, (long)0, Constants.SYSTEM_BUNDLE_LOCATION, bundleJarFile );

      bundleContext = new BundleContextImpl( cube, this );
      state = ACTIVE;

      // register package admin service
      Properties props = new Properties();
      props.put( "Description", "OSGi compliant package admin service by Opensugar" );
      bundleContext.registerService( PackageAdmin.class.getName(), cube.getPackageAdmin(), props );

      if ( cube.enforcesPermissions() ) {
         // register permission admin service
         props = new Properties();
         props.put( "Description", "OSGi compliant permission admin service by Opensugar" );
         bundleContext.registerService( PermissionAdmin.class.getName(), cube.getPermissionAdmin(), props );
      }
   }

   // redefinition of getBundleContext() method
   public BundleContextImpl getBundleContext() {
      return bundleContext;
   }

   // redefinition of getState() method
   public int getState() {
      return state;
   }

   public void start() {
      // Do nothing, framework already started
   }

   public void stop() {
      // shut down framework on another thread
      ( new Thread() { public void run() { getCube().shutdown( true ); state = RESOLVED; } } ).start();

      state = STOPPING;
      // return immediately
   }

   public void update() {
      // stop framework
      getCube().shutdown( false );

      int threadCount = Thread.activeCount() - 1;
      if ( threadCount > 0 ) {
         getCube().log( getCube().LOG_WARNING, "Framework is stopped but " + threadCount + " bundle threads are still running\n         Some bundles are not shutting down all their threads when stopping" );
      }

      // restart framework
      getCube().startup( null );
   }

   public void update( InputStream in ) throws BundleException {
      // should flash cube
      throw new BundleException( "Cube flashing not implemented yet" );
   }

   public void uninstall() throws BundleException {
      throw new BundleException( "System Bundle cannot be uninstalled" );
   }

}
