/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube.java2;

import java.util.Hashtable;
import java.util.zip.ZipFile;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;

// This class is used for interoperability both on Java 1 and 2.
// In Java 2, it would be better to use the classes provided by Java to parse
// the jar file manifest.
// These utilities however are not available in Java 1.1, so we had to develop
// this class.
public class ManifestHelper {

   private static final String MANIFEST_FILE = "META-INF/MANIFEST.MF";

   // manifest main attributes
   private Hashtable mainAttributes = new Hashtable();

   public ManifestHelper( ZipFile archive ) throws IOException {
      // in java 2, could do something like this, though we probably would use the
      // Attributes class without converting it to a Properties.
      /*
      java.util.jar.Manifest manifest = new java.util.jar.Manifest( archive.getInputStream( archive.getEntry( MANIFEST_FILE ) ) );
      java.util.jar.Attributes attr = manifest.getMainAttributes();
      java.util.Iterator iter = attr.keySet().iterator();
      String key;
      while ( iter.hasNext() ) {
         key = iter.next().toString();
         mainAttributes.put( key, attr.getValue( key ) );
      }
      */
      parseManifest( archive );
   }

   // I am not at all familiar with the augmented Backus-Naur Form. So I've done what
   // seems sensible here, rather than go look up what should really be done. As a
   // result, this method is probably is too laxist on manifest syntax (no limit on
   // line length for instance) but I'd rather accept too much than too little.
   // This method may need updating if there are valid syntax cases that I have not
   // taken into account.
   private void parseManifest( ZipFile archive ) throws IOException {
      // ISO-8859-1 is the encoding that works, don't ask me why!
      BufferedReader br = new BufferedReader( new InputStreamReader( archive.getInputStream( archive.getEntry( MANIFEST_FILE ) ), "ISO-8859-1" ) );
      String attributeName = null;
      String attributeValue = null;
      boolean done = false;
      int lineNumber = 0;

      // main attributes are the manifest headers and values from the main section
      // of the bundle's manifest file; that is, all lines prior to the first blank
      // line
      while ( !done ) {
         String line = br.readLine();
         lineNumber++;
         if ( line == null || line.trim().length() == 0 ) {
            // null line: we have reached the end of the manifest
            // empty line: we have reached the end of the main section of the bundle's manifest

            // record last attribute
            if ( attributeName != null ) {
               mainAttributes.put( attributeName, attributeValue );
            }

            // we are done
            done = true;
         }
         else if ( !line.startsWith( " " ) ) {
            // this line starts a new attribute

            // record last attribute
            if ( attributeName != null ) {
               mainAttributes.put( attributeName, attributeValue );
            }

            // proceed to new attribute
            int n = line.indexOf( ": " );
            if ( n == -1 || n + 2 >= line.length() ) {
               throw new IOException( "Invalid line in bundle manifest (line " + lineNumber + "): " + line );
            }
            else {
               attributeName = line.substring( 0, n );
               attributeValue = line.substring( n + 2, line.length() );
            }
         }
         else {
            // this line is a continuation line
            if ( attributeValue == null ) {
               throw new IOException( "Misplaced continuation line in bundle manifest (line " + lineNumber + "): " + line );
            }
            else {
               attributeValue = attributeValue + line.substring( 1 );
            }
         }
      }
   }

   public Hashtable getMainAttributes() {
      return mainAttributes;
   }

}
