/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube.ldap;

import java.util.Dictionary;

// checks string proximity (case-insensitive, ignores white spaces)
public class ApproxClause extends SimpleClause {

   public ApproxClause( String key, String value, boolean keysCaseSensitive ) {
      super( key, value, keysCaseSensitive );
   }

   public boolean filter( Dictionary props ) {
      Object obj = getProperty( getKey(), props );
      if ( obj == null ) {
         return false;
      }

      if ( obj instanceof String ) {
         return removeWhiteSpace( (String)obj ).toLowerCase().equals( removeWhiteSpace( getValue() ).toLowerCase() );
      }
      else if ( obj instanceof String[] ) {
         String[] s = (String[])obj;
         for ( int i = 0; i < s.length; i++ ) {
            if ( s[ i ] != null && removeWhiteSpace( s[ i ] ).toLowerCase().equals( removeWhiteSpace( getValue() ).toLowerCase() ) ) {
               return true;
            }
         }
      }
      return false;
   }

   private static String removeWhiteSpace( String s ) {
      StringBuffer sb = new StringBuffer();
      char c;
      for ( int i = 0; i < s.length(); i++ ) {
         c = s.charAt( i );
         if ( !Character.isWhitespace( c ) ) {
            sb.append( c );
         }
      }
      return sb.toString();
   }

   public String getCanonicalForm() {
      return "(" + getKey() + Filter.APPROX + getValue() + ")";
   }

}
