/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube.ldap;

import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Vector;

public abstract class Clause {

   public static final double EPS = 1e-6;

   public abstract boolean filter( Dictionary props );
   public abstract boolean areKeysCaseSensitive();
   public abstract String getCanonicalForm();

   protected Object getProperty( String key, Dictionary props ) {
      if ( props == null ) {
         return null;
      }

      if ( areKeysCaseSensitive() ) {
         return props.get( key );
      }
      else {
         Enumeration keys = props.keys();
         String k;
         while ( keys.hasMoreElements() ) {
            k = (String)keys.nextElement();
            if ( k.equalsIgnoreCase( key ) ) {
               return props.get( k );
            }
         }
         return null;
      }
   }

   protected static int compare( Object obj, String value ) {
      // Treat strings
      if ( obj instanceof String ) {
         if ( obj.toString().equals( value ) ) {
            return 0;
         }
         else {
            return Integer.MAX_VALUE;
         }
      }
      // Treat numbers
      else if ( obj instanceof Number ) {
         try {
            double d1 = ( (Number)obj ).doubleValue();
            double d2 = Double.valueOf( value ).doubleValue();
            if ( Math.abs( d1 - d2 ) < EPS ) {
              return 0;
            }
            else if ( d1 < d2 ) {
              return -1;
            }
            else {
              return 1;
            }
         }
         catch( NumberFormatException ignore ) { // go to end
         }
      }
      // Treat characters
      else if ( obj instanceof Character ) {
         if ( value.length() == 1 ) {
            char c1 = ( (Character)obj ).charValue();
            char c2 = value.charAt( 0 );
            return (int)c1 - (int)c2;
         }
      }
      // Treat booleans
      else if ( obj instanceof Boolean ) {
         Boolean b1 = (Boolean)obj;
         Boolean b2 = Boolean.valueOf( value );
         if ( b1.equals( b2 ) ) {
            return 0;
         }
      }
      return Integer.MAX_VALUE;
   }

   protected int[] compareSets( Object obj, String value ) {
      // Treat arrays
      if ( obj instanceof Object[] ) {
         Object[] array = (Object[])obj;
         int[] ret = new int[ array.length ];
         for ( int i = 0; i < array.length; i++ ) {
            ret[ i ] = compare( array[ i ], value );
         }
         return ret;
      }
      // Treat vectors
      else if ( obj instanceof Vector ) {
         Vector vector = (Vector)obj;
         int[] ret = new int[ vector.size() ];
         for ( int i = 0; i < vector.size(); i++ ) {
            ret[ i ] = compare( vector.elementAt( i ), value );
         }
         return ret;
      }

      int[] ret = new int[ 1 ];
      ret[ 0 ] = Integer.MAX_VALUE;
      return ret;
   }

}