/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube.ldap;

import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.Constants;

import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

public class LDAPFilter implements org.osgi.framework.Filter {

   private String filterString;
   private Clause clause;

   public LDAPFilter( String filterString, boolean keysCaseSensitive ) throws InvalidSyntaxException {
      this.filterString = filterString;
      clause = Filter.parse( filterString, keysCaseSensitive );
   }

   public String getFilterString() {
      return filterString;
   }

   public boolean equals( Object obj ) {
      return filterString.equals( obj );
   }

   public int hashCode() {
      return filterString.hashCode();
   }

   public boolean match( Dictionary dictionary ) throws IllegalArgumentException {
      // Throw IllegalArgumentException if dictionary contains case variants of
      // the same key name
      Enumeration keys = dictionary.keys();
      Vector tmp = new Vector();
      String key;
      while ( keys.hasMoreElements() ) {
         key = (String)keys.nextElement();
         tmp.addElement( key.toLowerCase() );
      }
      while ( tmp.size() > 0 ) {
         key = (String)tmp.elementAt( 0 );
         tmp.removeElementAt( 0 );
         if ( tmp.indexOf( key ) != -1 ) {
            throw new IllegalArgumentException( "Dictionary contains case variants of the same key name: " + key );
         }
      }

      return clause.filter( dictionary );
   }

   public boolean match( ServiceReference reference ) {
      Hashtable props = new Hashtable();
      String[] keys = reference.getPropertyKeys();
      for ( int i = 0; i < keys.length; i++ ) {
         props.put( keys[ i ], reference.getProperty( keys[ i ] ) );
      }
      return clause.filter( props );
   }

   public String toString() {
      return clause.getCanonicalForm();
   }

}