/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube.ldap;

import java.util.Dictionary;
import java.util.StringTokenizer;

public class SubstringClause extends SimpleClause {

   public static final char WILDCARD = '*';

   public SubstringClause( String key, String value, boolean keysCaseSensitive ) {
      super( key, value, keysCaseSensitive );
   }

   public boolean filter( Dictionary props ) {
      Object obj = getProperty( getKey(), props );
      if ( obj == null ) {
         return false;
      }

      if ( obj instanceof String ) {
         return isSubstring( (String)obj, getValue() );
      }
      return false;
   }

   private static boolean isSubstring( String s, String pattern ) {
      StringTokenizer tokens = new StringTokenizer( pattern, "" + WILDCARD, true );
      String token;
      int position = 0;
      while ( tokens.hasMoreTokens() ) {
         token = tokens.nextToken();
         if ( token.equals( "" + WILDCARD ) ) {
            if ( tokens.hasMoreTokens() ) {
               token = tokens.nextToken();
            }
            else {
               return true;
            }
         }
         position = s.indexOf( token, position );
         if ( position == -1 ) {
            return false;
         }
         position += token.length();
      }
      if ( position == s.length() ) {
         return true;
      }
      return false;
   }

   public String getCanonicalForm() {
      return "(" + getKey() + Filter.EQUALS + getValue() + ")";
   }

}
