/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube.packageAdmin;

import com.opensugar.cube.BundleClassLoader;

import org.osgi.service.packageadmin.ExportedPackage;
import org.osgi.framework.Bundle;

import java.util.Vector;
import java.net.URL;

public class ExportedPackageImpl implements ExportedPackage, HasVersion {

   private String name;
   private String specificationVersion;
   private Bundle exportingBundle;
   private BundleClassLoader classLoader;
   private Vector importingBundles;
   private boolean removalPending;
   private boolean open;

   public ExportedPackageImpl( String name, String specificationVersion, Bundle exportingBundle, BundleClassLoader classLoader ) {
      this.name = name;
      this.specificationVersion = specificationVersion;
      this.exportingBundle = exportingBundle;
      this.classLoader = classLoader;

      importingBundles = new Vector();
      importingBundles.addElement( exportingBundle );
      removalPending = false;
      open = false;
   }

   public String getName() {
      return name;
   }

   public String getSpecificationVersion() {
      return specificationVersion;
   }

   public Bundle getExportingBundle() {
      return exportingBundle;
   }

   public Bundle[] getImportingBundles() {
      Vector clone = (Vector)importingBundles.clone();
      Bundle[] ret = new Bundle[ clone.size() ];
      clone.copyInto( ret );
      return ret;
   }

   public boolean isRemovalPending() {
      return removalPending;
   }

   protected void addImportingBundle( Bundle bundle ) {
      if ( importingBundles.indexOf( bundle ) == -1 ) {
         importingBundles.addElement( bundle );
      }
   }

   protected void removeImportingBundle( Bundle bundle ) {
      // a bundle that exports a package implicitly imports it
      // therefore the spec says that getImportingBundles should return a list that
      // always contains the exporting bundle
      // but the spec also says the getImportingBundles method should return resolved
      // bundles that import the package
      //
      // therefore, I think it is ok to remove the exporting bundle from the list of
      // importers if the exporting bundle is being uninstalled
      // if the exporting bundle is being updated, then if its updated version exports
      // the same package version again, it will go back immediately in the list of
      // importing bundles, so the removal will be very brief and will pause no problems
      // if the exporting bundle is being updated, and its updated version exports a
      // larger version of the package, it is correct that the bundle should not be
      // in the list of importers of this package because it will not be resolved...
      //
      // the spec seems to require some clarification here.
      //
      // in the meantime, I will allow the exporting package to be removed from the list
      // of importers, and leave it to PackageAdminImpl to take care when calling this
      // method
      //
      //if ( bundle.getBundleId() == exportingBundle.getBundleId() && bundle.getState() != bundle.UNINSTALLED ) {
      //   throw new IllegalArgumentException( "Package exporter cannot be removed from the package's list of importers (unless the package exporter is uninstalled)" );
      //}

      importingBundles.removeElement( bundle );
   }

   protected boolean isImportingBundle( Bundle bundle ) {
      return ( importingBundles.indexOf( bundle ) != -1 );
   }

   protected void setRemovalPending() {
      removalPending = true;
   }

   protected void setStale() {
      exportingBundle = null;
      classLoader = null;
      importingBundles = null;
      removalPending = true ;
   }

   protected boolean isOpen() {
      return open;
   }

   protected Class loadClass( String className ) throws ClassNotFoundException {
      try {
         Class clazz = classLoader.loadOwnClass( className );
         open = true;
         return clazz;
      }
      catch ( ClassNotFoundException e ) {
         throw e;
      }
   }

   protected URL getResource( String name ) {
      URL url = classLoader.getOwnResource( name );
      if ( url != null ) {
         open = true;
      }
      return url;
   }

}
