/*
 * JEFFREE: Java(TM) Embedded Framework FREE
 * Copyright (C) 1999-2003 - Opensugar
 *
 * The contents of this file are subject to the Jeffree Public License,
 * as defined by the file JEFFREE_LICENSE.TXT 
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the Objectweb web site
 * (www.objectweb.org). 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License. 
 * 
 * The Original Code is JEFFREE, including the java package com.opensugar.cube,
 * released January 1, 2003.
 * 
 * The Initial Developer of the Original Code is Opensugar.
 * The Original Code is Copyright Opensugar.
 * All Rights Reserved.
 *
 * Initial developer(s): Pierre Scokaert (Opensugar)
 * Contributor(s):
 */

package com.opensugar.cube.simple;

import com.opensugar.cube.BundleImpl;
import com.opensugar.cube.BundleClassLoader;
import com.opensugar.cube.Util;
import com.opensugar.cube.NamedPropertySet;
import com.opensugar.cube.java2.FileHelper;

import org.osgi.framework.FrameworkEvent;

import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;

import java.security.SecureClassLoader;
import java.security.CodeSource;

public class SecureBundleClassLoader extends SecureClassLoader implements BundleClassLoader {

   private BundleImpl bundle;
   private ZipFile[] classPathJarFiles;   // list of jar files on the class path in the order that they must be searched to find class
   private Hashtable nativeLibraries;     // library name --> file path (local copy of native library file)
   private Hashtable classCache;          // cache for loader classes: class name --> class
   private Hashtable resourceURLCache;    // cache for loader resource urls: resource name --> url

   private ZipFile mainArchive;           // should not be used except in clean-up

   private CodeSource codeSource;

   protected SecureBundleClassLoader( BundleImpl bundle, File bundleJarFile, NamedPropertySet[] nativeLibs, NamedPropertySet[] classPath ) throws IOException {
      super( SecureBundleClassLoader.class.getClassLoader() );

      if ( bundle.getBundleId() == 0 ) {
         codeSource = new CodeSource( new URL( "file:lib/main.jar" ), null );
      }
      else {
         try {
            codeSource = new CodeSource( new URL( bundle.getLocation() ), null );
         }
         catch ( MalformedURLException e ) {
            System.out.println( "Bundle with invalid location: " + bundle.getLocation() );
            codeSource = new CodeSource( new URL( "file://null" ), null );
         }
      }

      initialize( bundle, bundleJarFile, nativeLibs, classPath );
   }

   private static boolean isClassLoaderJava2() {
      return true;
   }

   private void initialize( BundleImpl bundle, File bundleJarFile, NamedPropertySet[] nativeLibs, NamedPropertySet[] classPath ) throws IOException {
      this.bundle = bundle;

      mainArchive = new ZipFile( bundleJarFile );

      Vector classPathFilesVector = new Vector();
      if ( classPath == null || classPath.length == 0 ) {
         // If class path is null or empty, include only bundle jar file in this class loader's archives
         classPathFilesVector.addElement( bundleJarFile );
      }
      else {
         // Add jar files in the order specified in the bundle class path manifest
         // header
         String classPathItem;
         ZipEntry entry;
         File innerJarFileDir;
         String innerJarFileName;
         File innerJarFile;
         int n;
         for ( int i = 0; i < classPath.length; i++ ) {
            classPathItem = classPath[ i ].getName().trim();
            if ( classPathItem.equals( "." ) ) {
               // If class path item is ".", then add the bundle jar to the class path
               classPathFilesVector.addElement( bundleJarFile );
            }
            else {
               // If class path item is not ".", it must be the name of an inner jar file
               // contained in the bundle jar file.
               // Add the inner jar file to the class path
               entry = mainArchive.getEntry( classPathItem );
               if ( entry != null ) {
                  innerJarFileDir = bundle.getCube().getBundleInnerFilesDirectory( bundle.getBundleId() );
                  innerJarFileName = classPathItem;
                  n = innerJarFileName.indexOf( '/' );
                  while ( n > 0 && n < innerJarFileName.length() ) {
                     innerJarFileDir = new File( innerJarFileDir, innerJarFileName.substring( 0, n ) );
                     innerJarFileDir.mkdir();
                     innerJarFileName = innerJarFileName.substring( n + 1 );
                     n = innerJarFileName.indexOf( '/' );
                  }
                  innerJarFile = new File( innerJarFileDir, innerJarFileName );
                  Util.transferData( mainArchive.getInputStream( entry ), new FileOutputStream( innerJarFile ) );
                  classPathFilesVector.addElement( innerJarFile );
               }
               else {
                  // ignore missing inner jar file
                  //throw new FileNotFoundException( "Missing inner jar in bundle jar: " + classPathItem );
               }
            }
         }
      }
      classPathJarFiles = new ZipFile[ classPathFilesVector.size() ];
      for ( int i = 0; i < classPathFilesVector.size(); i++ ) {
         classPathJarFiles[ i ] = new ZipFile( (File)classPathFilesVector.elementAt( i ) );
      }

      nativeLibraries = new Hashtable();
      if ( nativeLibs != null ) {
         String nativeLibName;
         ZipEntry entry;
         File innerFileDir;
         File innerFile;
         int n;
         for ( int i = 0; i < nativeLibs.length; i++ ) {
            nativeLibName = nativeLibs[ i ].getName();
            entry = mainArchive.getEntry( nativeLibName );
            if ( entry != null ) {
               innerFileDir = bundle.getCube().getBundleInnerFilesDirectory( bundle.getBundleId() );
               //innerFileName = nativeLibName;
               n = nativeLibName.indexOf( '/' );
               while ( n > 0 && n < nativeLibName.length() ) {
                  innerFileDir = new File( innerFileDir, nativeLibName.substring( 0, n ) );
                  innerFileDir.mkdir();
                  nativeLibName = nativeLibName.substring( n + 1 );
                  n = nativeLibName.indexOf( '/' );
               }
               innerFile = new File( innerFileDir, nativeLibName );
               int index = 2;
               while ( innerFile.exists() ) {
                  innerFile = new File( innerFileDir, nativeLibName + index );
                  index++;
               }
               Util.transferData( mainArchive.getInputStream( entry ), new FileOutputStream( innerFile ) );
               String nativeLibPath = innerFile.getAbsolutePath();
               registerNativeLibrary( nativeLibName, nativeLibPath );
            }
            else {
               // ignore missing native library
               //throw new FileNotFoundException( "Missing native library in bundle jar: " + nativeLibName );
            }
         }
      }

      classCache = new Hashtable();
      resourceURLCache = new Hashtable();
   }

   private void registerNativeLibrary( String nativeLibName, String nativeLibPath ) {
      // register library with library name
      nativeLibraries.put( nativeLibName, nativeLibPath );
      // register library with library name without extensions
      int index = nativeLibName.indexOf( "." );
      if ( index != -1 ) {
         nativeLibraries.put( nativeLibName.substring( 0, index ), nativeLibPath );
      }

      // register library without directory info
      index = nativeLibName.lastIndexOf( "/" );
      if ( index != -1 ) {
         nativeLibraries.put( nativeLibName.substring( index + 1 ), nativeLibPath );
      }

      // register library without lib at beginning
      if ( nativeLibName.startsWith( "lib" ) && nativeLibName.length() > 3 ) {
         nativeLibraries.put( nativeLibName.substring( 3 ), nativeLibPath );
      }

      // copy native library to library path directory
      String nativeLibraryDir = System.getProperty( "com.opensugar.cube.library.dir", "nativeLibs" );
      File dir = new File( nativeLibraryDir );
      if ( !dir.exists() ) {
         dir.mkdirs();
      }
      try {
         Util.transferData( new FileInputStream( new File( nativeLibPath ) ), new FileOutputStream( new File( dir, nativeLibName ) ) );
      }
      catch( IOException e ) {
         bundle.getCube().log( bundle.getCube().LOG_ERROR, "Cannot create extract native library file from bundle jar to disk", e );
      }
   }

   // Library loading in JDK 1.2
   public String findLibrary( String name ) {
      return (String)nativeLibraries.get( name );
   }

   public Class loadClass( String name ) throws ClassNotFoundException {
      return loadClass( name, true );
   }

   protected Class loadClass( String name, boolean resolve ) throws ClassNotFoundException {
      name = processClassName( name );

      Class clazz = null;

      // First, try to load class from system class loader
      try {
         if ( isClassLoaderJava2() ) {
            clazz = getParent().loadClass( name );
         }
         else {
            clazz = findSystemClass( name );
         }
      }
      catch( ClassNotFoundException ignore ) {}

      // If class not found by system class loader, try to load class from imported
      // package  (after checking caller has appropriate IMPORT PackagePermission)
      if ( clazz == null ) {
         int n = name.lastIndexOf( "." );
         if ( n != -1 ) {
            // no need to check package permission
            // BundleImpl.initializeBundle() is where package exports and imports are
            // initialized, and a package import is only allowed if required package
            // permission exists
            try {
               clazz = bundle.getCube().getPackageAdmin().loadClass( bundle, name );
            }
            catch ( ClassNotFoundException ignore ) {}
         }
      }

      // If class not found either by system class loader or from imported package
      // load class from the class path of the bundle that owns this class loader
      if ( clazz == null ) {
         clazz = loadOwnClass( name );
      }

      return clazz;
   }

   public URL getResource( String name ) {
      name = processResourceName( name );

      URL url = null;

      // First, try to get resource from system class loader
      if ( isClassLoaderJava2() ) {
         url = getParent().getResource( name );
      }
      else {
         url = super.getResource( name );
      }

      // If resource not found by system class loader, try to get resource from imported
      // package  (after checking caller has appropriate IMPORT PackagePermission)
      if ( url == null ) {
         int n = name.lastIndexOf( "/" );
         if ( n != -1 ) {
            // no need to check package permission
            // BundleImpl.initializeBundle() is where package exports and imports are
            // initialized, and a package import is only allowed if required package
            // permission exists
            url = bundle.getCube().getPackageAdmin().getResource( bundle, name );
         }
      }

      // If resource not found either by system class loader or from imported package
      // get resource from the class path of the bundle that owns this class loader
      if ( url == null ) {
         url = getOwnResource( name );
      }

      return url;
   }

   public InputStream getResourceAsStream( String name ) {
      URL url = getResource( name );
      if ( url == null ) {
         return null;
      }
      else {
         try {
            if ( url.toString().startsWith( "file://" ) ) {
               String filename = url.toString();
               filename = filename.substring( 7, filename.length() );
               return new FileInputStream( filename );
            }
            return url.openStream();
         }
         catch( IOException e ) {
            return null;
         }
      }
   }

   private String processClassName( String name ) {
      // Sun VMs call this with '.' separators in the class name,
      // but Perc VM calls this method with '/' separators
      return name.replace( '/', '.' );
   }

   private String processResourceName( String name ) {
      // Sun VMs seem to make no difference whether or not the name
      // starts with "/",
      // but Perc does not find the jar file entry if its name starts with "/".
      // So, we strip the starting "/" if there is one.
      if ( name.startsWith( "/" ) && name.length() > 1 ) {
         name = name.substring( 1, name.length() );
      }

      return name;
   }

   public Class loadOwnClass( String name ) throws ClassNotFoundException {
      Class clazz = (Class)classCache.get( name );
      if ( clazz == null ) {
         clazz = defineClass( name );
         classCache.put( name, clazz );
      }
      return clazz;
   }

   public URL getOwnResource( String name ) {
      URL url = (URL)resourceURLCache.get( name );
      if ( url == null ) {
         url = defineResourceURL( name );
         if ( url != null ) {
            resourceURLCache.put( name, url );
         }
      }
      return url;
   }

   public void cleanUp() throws IOException {
      for ( int i = 0; i < classPathJarFiles.length; i++ ) {
         if ( !classPathJarFiles[ i ].equals( mainArchive ) ) {
            classPathJarFiles[ i ].close();
         }
      }
      mainArchive.close();

      classPathJarFiles = null;
   }

   public Object getCodeSource() {
      return codeSource;
   }

   private Class defineClass( String name ) throws ClassNotFoundException {
      String classFilePath = name.replace( '.', '/' ) + ".class";

      byte[] bytes;
      // search for class in class path jar files, in the order specified by the
      // bundle's class path manifest header value
      for ( int i = 0; i < classPathJarFiles.length; i++ ) {
         try {
            bytes = getJarEntryContents( classPathJarFiles[ i ], classFilePath );
            if ( bytes != null ) {
               Class clazz = defineClass( name, bytes, 0, bytes.length, codeSource );
               return clazz;
            }
         }
         catch( IOException e ) {
            bundle.getCube().fireFrameworkEvent( FrameworkEvent.ERROR, bundle, e );
         }
      }
      throw new ClassNotFoundException( name );
   }

   private URL defineResourceURL( String name ) {
      // search for resource in class path jar files, in the order specified by the
      // bundle's class path manifest header value
      for ( int i = 0; i < classPathJarFiles.length; i++ ) {
         URL url = getJarEntryURL( classPathJarFiles[ i ], name );
         if ( url != null ) {
            return url;
         }
      }
      return null;
   }

   // Get a url reference for a named entry inside one of this class loader's archives.
   private static int unjarCount = 0;
   private static Hashtable unjarred = new Hashtable();
   private URL getJarEntryURL( ZipFile archive, String entryName ) {
      if ( archive.getEntry( entryName ) != null ) {
         try {
            // This causes a MalformedURLException
            //url = new URL( "jar:" + FileHelper.toURL( file ) + "!/" + entryName );

            // I don't know how to solve the problem, so unjar the file and return
            // the url of the uncompressed file
            String key = archive.getName() + "." + entryName;
            File tmp;
            if ( unjarred.get( key ) == null ) {
               tmp = new File( getUnjarDirectory(), "unjar" + unjarCount++ );
               FileOutputStream fos = new FileOutputStream( tmp );
               fos.write( getJarEntryContents( archive, entryName ) );
               fos.close();
               unjarred.put( key, tmp );
            }
            tmp = (File)unjarred.get( key );
            return FileHelper.toURL( tmp );
         }
         catch( MalformedURLException e ) {
            bundle.getCube().log( bundle.getCube().LOG_ERROR, "Error getting jar entry url", e );
         }
         catch( IOException e ) {
            bundle.getCube().log( bundle.getCube().LOG_ERROR, "Error getting jar entry url", e );
         }
      }
      return null;
   }

   private File unjarDirectory;
   private synchronized File getUnjarDirectory() {
      if ( unjarDirectory == null ) {
         // this is the first time we get the directory
         // create the directory
         unjarDirectory = bundle.getBundleContext().getDataFile( "temp/unjar" );
         if ( unjarDirectory.exists() ) {
            // clean up old stuff from previous engine runs
            Util.recursiveFileDelete( unjarDirectory );
         }
         // some virtual machines seem to have a problem recursively creating directories
         // (e.g. personal java 3.1 for windows from sun). Give them some help
         unjarDirectory = bundle.getBundleContext().getDataFile( "temp" );
         unjarDirectory.mkdirs();
         unjarDirectory = bundle.getBundleContext().getDataFile( "temp/unjar" );
         unjarDirectory.mkdirs();
      }
      return unjarDirectory;
   }

   // Get a byte array containing the contents of a named entry from one of this class
   // loader's archives.
   private byte[] getJarEntryContents( ZipFile archive, String entryName ) throws IOException {
      ZipEntry entry = archive.getEntry( entryName );
      if ( entry != null ) {
         int entrySize = (int)entry.getSize();
         if ( entrySize < 0 ) {
            throw new IOException( "Unknown size for " + entry );
         }
         InputStream is = archive.getInputStream( entry );
         byte[] bytes = new byte[ entrySize ];
         DataInputStream dis = new DataInputStream( is );
         dis.readFully( bytes );
         is.close();
         return bytes;
      }
      return null;
   }

}

