/*
 *
 * Copyright 1999-2002 Opensugar S.A. All Rights Reserved.
 *
 * This software is the proprietary information of Opensugar S.A.
 * Use is subject to license terms.
 *
 */

/*
 *
 * Dummy implementation of parts of the java.security package, for JVMs that do
 * not support permissions.
 *
 * The package java.security is exported by the system bundle.
 *
 * When classes are loaded by the OSGi framework, they are first searched for using
 * the system class loader. If they are found using the system class loader, they are
 * not searched for in the shared packages. Therefore, the dummy implementation classes
 * will never be loaded if the JVM does support permissions.
 * If the JVM does not support permissions, then classes in the java.security package
 * will not be found when searched for using the system class loader; they will then
 * be searched for in the shared packages and will be found in the packages exported by
 * the system bundle.
 * (Note: the package sharing mechanism of the cube has been written with a special case
 * for classes in the java.security package, so that bundles may load classes in this
 * package from shared packages, even the bundle has no import clause for java.security
 * in its manifest. In other words, java.security is implicitly imported by all bundles.)
 *
 *
 */

package java.security;

public class AccessController {

   public static void checkPermission( Permission permission ) {
      // do nothing
   }

   public static Object doPrivileged( PrivilegedAction privilegedAction ) {
      return privilegedAction.run();
   }

   public static Object doPrivileged( PrivilegedAction privilegedAction, AccessControlContext accessControlContext ) {
      return privilegedAction.run();
   }

   public static Object doPrivileged( PrivilegedExceptionAction privilegedExceptionAction ) throws PrivilegedActionException {
      try {
         return privilegedExceptionAction.run();
      }
      catch ( Exception e ) {
         throw new PrivilegedActionException( e );
      }
   }

   public static Object doPrivileged( PrivilegedExceptionAction privilegedExceptionAction, AccessControlContext accessControlContext ) throws PrivilegedActionException {
      try {
         return privilegedExceptionAction.run();
      }
      catch ( Exception e ) {
         throw new PrivilegedActionException( e );
      }
   }

   public static AccessControlContext getContext() {
      return null;
   }

}