/*
 *
 * Copyright 1999-2002 Opensugar S.A. All Rights Reserved.
 *
 * This software is the proprietary information of Opensugar S.A.
 * Use is subject to license terms.
 *
 */

/*
 *
 * Dummy implementation of parts of the java.security package, for JVMs that do
 * not support permissions.
 *
 * The package java.security is exported by the system bundle.
 *
 * When classes are loaded by the OSGi framework, they are first searched for using
 * the system class loader. If they are found using the system class loader, they are
 * not searched for in the shared packages. Therefore, the dummy implementation classes
 * will never be loaded if the JVM does support permissions.
 * If the JVM does not support permissions, then classes in the java.security package
 * will not be found when searched for using the system class loader; they will then
 * be searched for in the shared packages and will be found in the packages exported by
 * the system bundle.
 * (Note: the package sharing mechanism of the cube has been written with a special case
 * for classes in the java.security package, so that bundles may load classes in this
 * package from shared packages, even the bundle has no import clause for java.security
 * in its manifest. In other words, java.security is implicitly imported by all bundles.)
 *
 *
 */

package java.security;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.IOException;

public abstract class BasicPermission extends Permission implements Serializable {

   private transient boolean wildcard;
   private transient String path;

   public BasicPermission(String s) {
      super( s );
      init( s );
   }

   public BasicPermission( String s, String s1 ) {
      super( s );
      init( s );
   }

   public boolean equals( Object obj ) {
      if( obj == this ) {
         return true;
      }
      if ( obj == null || obj.getClass() != getClass() ) {
         return false;
      }
      else {
         BasicPermission basicpermission = (BasicPermission)obj;
         return getName().equals( basicpermission.getName() );
      }
   }

   public String getActions() {
      return "";
   }

   public int hashCode() {
      return getName().hashCode();
   }

   public boolean implies( Permission permission ) {
      if ( permission == null || permission.getClass() != getClass() ) {
         return false;
      }
      BasicPermission basicpermission = (BasicPermission)permission;
      if( wildcard ) {
         if( basicpermission.wildcard ) {
            return basicpermission.path.startsWith( path );
         }
         else {
            return basicpermission.path.length() > path.length() && basicpermission.path.startsWith( path );
         }
      }
      if ( basicpermission.wildcard ) {
         return false;
      }
      else {
         return path.equals( basicpermission.path );
      }
   }

   private void init(String s) {
      if( s == null ) {
         throw new IllegalArgumentException( "name can't be null" );
      }
      if( s.endsWith( ".*" ) || s.equals( "*" ) ) {
         wildcard = true;
         if( s.length() == 1 ) {
            path = "";
         }
         else {
            path = s.substring( 0, s.length() - 1 );
         }
      }
      else {
         path = s;
      }
   }

   public PermissionCollection newPermissionCollection() {
      return new BasicPermissionCollection();
   }

   private synchronized void readObject( ObjectInputStream objectinputstream ) throws IOException, ClassNotFoundException {
      objectinputstream.defaultReadObject();
      init( getName() );
   }

}
