/*
 *
 * Copyright 1999-2002 Opensugar S.A. All Rights Reserved.
 *
 * This software is the proprietary information of Opensugar S.A.
 * Use is subject to license terms.
 *
 */

/*
 *
 * Dummy implementation of parts of the java.security package, for JVMs that do
 * not support permissions.
 *
 * The package java.security is exported by the system bundle.
 *
 * When classes are loaded by the OSGi framework, they are first searched for using
 * the system class loader. If they are found using the system class loader, they are
 * not searched for in the shared packages. Therefore, the dummy implementation classes
 * will never be loaded if the JVM does support permissions.
 * If the JVM does not support permissions, then classes in the java.security package
 * will not be found when searched for using the system class loader; they will then
 * be searched for in the shared packages and will be found in the packages exported by
 * the system bundle.
 * (Note: the package sharing mechanism of the cube has been written with a special case
 * for classes in the java.security package, so that bundles may load classes in this
 * package from shared packages, even the bundle has no import clause for java.security
 * in its manifest. In other words, java.security is implicitly imported by all bundles.)
 *
 *
 */

package java.security;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;

final class BasicPermissionCollection extends PermissionCollection implements Serializable {

   private Hashtable permissions;
   private boolean all_allowed;

   public BasicPermissionCollection() {
      permissions = new Hashtable( 11 );
      all_allowed = false;
   }

   public void add(Permission permission) {
      if( !( permission instanceof BasicPermission ) ) {
         throw new IllegalArgumentException( "invalid permission: " + permission );
      }
      BasicPermission basicpermission = (BasicPermission)permission;
      permissions.put( basicpermission.getName(), permission );
      if( !all_allowed && basicpermission.getName().equals( "*" ) ) {
         all_allowed = true;
      }
   }

   public Enumeration elements() {
      return permissions.elements();
   }

   public boolean implies(Permission permission) {
      if( !( permission instanceof BasicPermission ) ) {
         return false;
      }
      BasicPermission basicpermission = (BasicPermission)permission;
      if( all_allowed ) {
         return true;
      }
      String s = basicpermission.getName();
      Permission permission1 = (Permission)permissions.get( s );
      if ( permission1 != null ) {
         return permission1.implies( permission );
      }
      int i;
      for( int j = s.length() - 1; ( i = s.lastIndexOf( ".", j ) ) != -1; j = i - 1 ) {
         s = s.substring( 0, i + 1 ) + "*";
         Permission permission2 = (Permission)permissions.get( s );
         if( permission2 != null ) {
            return permission2.implies( permission );
         }
      }

      return false;
   }

}
