/*
 *
 * Copyright 1999-2002 Opensugar S.A. All Rights Reserved.
 *
 * This software is the proprietary information of Opensugar S.A.
 * Use is subject to license terms.
 *
 */

/*
 *
 * Dummy implementation of parts of the java.security package, for JVMs that do
 * not support permissions.
 *
 * The package java.security is exported by the system bundle.
 *
 * When classes are loaded by the OSGi framework, they are first searched for using
 * the system class loader. If they are found using the system class loader, they are
 * not searched for in the shared packages. Therefore, the dummy implementation classes
 * will never be loaded if the JVM does support permissions.
 * If the JVM does not support permissions, then classes in the java.security package
 * will not be found when searched for using the system class loader; they will then
 * be searched for in the shared packages and will be found in the packages exported by
 * the system bundle.
 * (Note: the package sharing mechanism of the cube has been written with a special case
 * for classes in the java.security package, so that bundles may load classes in this
 * package from shared packages, even the bundle has no import clause for java.security
 * in its manifest. In other words, java.security is implicitly imported by all bundles.)
 *
 *
 */

package java.security;

import java.io.Serializable;

public abstract class Permission implements Guard, Serializable {

   private String name;

   public Permission( String s ) {
      name = s;
   }

   public void checkGuard( Object obj ) throws SecurityException {}

   public abstract boolean equals( Object obj );

   public abstract String getActions();

   public final String getName() {
      return name;
   }

   public abstract int hashCode();

   public abstract boolean implies( Permission permission );

   public PermissionCollection newPermissionCollection() {
      return null;
   }

   public String toString() {
      return "(" + getClass().getName() + " " + name + " " + getActions() + ")";
   }

}
